## 调用示例
### 1 合约调用
对数据有修改的操作如插入和删除，需要调用send方法。
对数据的查询操作如查询某个键的历史，需要调用query方法。
#### 1.1 调用简单value的合约
* 插入一对键值
```java
SdkClient sdkClient = new SdkClient(configFilePath);
Contract contract = new Contract(contractName, endorserNodes, consensusNode);
contract.send(sdkClient, "insert", new String[]{"a", "huaweichain"});
```

* 通过键查询值
```java
SdkClient sdkClient = new SdkClient(configFilePath);
Contract contract = new Contract(contractName, endorserNodes, consensusNode);
contract.query(sdkClient, "query", new String[]{"a"});
```

* 删除指定的键
```java
SdkClient sdkClient = new SdkClient(configFilePath);
Contract contract = new Contract(contractName, endorserNodes, consensusNode);
contract.send(sdkClient, "delete", new String[]{"a"});
```

* 查询指定键的历史
```java
SdkClient sdkClient = new SdkClient(configFilePath);
Contract contract = new Contract(contractName, endorserNodes, consensusNode);
contract.query(sdkClient, "queryKeyHistory", new String[]{"a"});
```

* 范围查询。两个参数是 startKey 和 endKey，左闭右开区间。 
  范围查询基于字符串的字典序比较，如果键包含数字编码，应提前规划好数字的总位数，不足的补前缀0
```java
SdkClient sdkClient = new SdkClient(configFilePath);
Contract contract = new Contract(contractName, endorserNodes, consensusNode);
contract.send(sdkClient, "insert", new String[]{"a000", "huaweichain"});
contract.send(sdkClient, "insert", new String[]{"a001", "huaweichain"});
contract.send(sdkClient, "insert", new String[]{"a019", "huaweichain"});
contract.send(sdkClient, "insert", new String[]{"a020", "huaweichain"});
contract.send(sdkClient, "insert", new String[]{"a021", "huaweichain"});
contract.query(sdkClient, "queryInRange", new String[]{"a000", "a020"});
```

* 条件查询。前两个参数是 startKey 和 endKey，第三个参数是判断符，第四个参数是要判断的值
```java
SdkClient sdkClient = new SdkClient(configFilePath);
Contract contract = new Contract(contractName, endorserNodes, consensusNode);
contract.send(sdkClient, "insert", new String[]{"c1", "1"});
contract.send(sdkClient, "insert", new String[]{"c2", "2"});
contract.send(sdkClient, "insert", new String[]{"c3", "3"});
contract.query(sdkClient, "queryInRangeMatchCondition", new String[]{"c1", "c4", ">", "1"});
```
#### 1.2 调用结构体value的合约
* 插入一个marble
```java
SdkClient sdkClient = new SdkClient(configFilePath);
Contract contract = new Contract(contractName, endorserNodes, consensusNode);
contract.send(sdkClient, "insertMarble", new String[]{"marble01", "zhangsan", "red", "10"});
```

* 查询一个marble
```java
SdkClient sdkClient = new SdkClient(configFilePath);
Contract contract = new Contract(contractName, endorserNodes, consensusNode);
contract.query(sdkClient, "queryMarble", new String[]{"marble01"});
```

* 删除一个marble
```java
SdkClient sdkClient = new SdkClient(configFilePath);
Contract contract = new Contract(contractName, endorserNodes, consensusNode);
contract.send(sdkClient, "deleteMarble", new String[]{"marble01"});
```

* 查询某个marble的历史
```java
SdkClient sdkClient = new SdkClient(configFilePath);
Contract contract = new Contract(contractName, endorserNodes, consensusNode);
contract.query(sdkClient, "queryMarbleKeyHistory", new String[]{"marble01"});
```

* 查询某个范围内的marble。两个参数是 startKey 和 endKey，左闭右开区间。 
  范围查询基于字符串的字典序比较，如果键中包含数字编码，应提前规划好数字的总位数，不足的补前缀0
```java
SdkClient sdkClient = new SdkClient(configFilePath);
Contract contract = new Contract(contractName, endorserNodes, consensusNode);
contract.query(sdkClient, "queryMarbleInRange", new String[]{"marble01", "marble03"});
```

* 插入marble的同时建立复合索引。复合索引的索引名称以color/size为例
```java
SdkClient sdkClient = new SdkClient(configFilePath);
Contract contract = new Contract(contractName, endorserNodes, consensusNode);
contract.send(sdkClient, "insertMarbleAndIndex", new String[]{"marble01", "zhangsan", "red", "10"});
```

* 通过复合索引查询满足条件的marble。复合索引的索引名称以color/size为例
```java
SdkClient sdkClient = new SdkClient(configFilePath);
Contract contract = new Contract(contractName, endorserNodes, consensusNode);
contract.query(sdkClient, "queryMarblesByIndex", new String[]{"red", "10"});
```

* 通过复合索引转移marble。复合索引的索引名称以color/size为例
```java
SdkClient sdkClient = new SdkClient(configFilePath);
Contract contract = new Contract(contractName, endorserNodes, consensusNode);
contract.send(sdkClient, "transferMarblesByIndex", new String[]{"red", "10", "lisi"});
```

* 删除某个marble和它对应的索引值。复合索引的索引名称以color/size为例
```java
SdkClient sdkClient = new SdkClient(configFilePath);
Contract contract = new Contract(contractName, endorserNodes, consensusNode);
contract.send(sdkClient, "deleteMarbleAndIndex", new String[]{"marble01"});
```

### 2 查询区块数据
#### 2.1 查询区块

* 根据区块号查询区块
```java
SdkClient sdkClient = new SdkClient(configFilePath);
BlockOuterClass.Block block = BlockUtil.queryBlockByNumber(sdkClient, 2);
```

* 查询当前最后一个区块
```java
SdkClient sdkClient = new SdkClient(configFilePath);
BlockOuterClass.Block block = BlockUtil.queryLastBlock(sdkClient);
```

* 根据交易ID查询区块
```java
SdkClient sdkClient = new SdkClient(configFilePath);
String txID = "xxxxxxxxxxxxxxxxxxxxxx";
BlockOuterClass.Block block = BlockUtil.queryBlockByTxID(sdkClient, txID);
```
#### 2.2 解析区块

* 打印区块包含的交易信息
```java
BlockUtil.printBlockTxsInfo(block);
```

* 打印区块基本信息
```java
BlockUtil.printBlockBasicInfo(block);
```

* 获取区块的交易ID列表
```java
List<String> txIdList = BlockUtil.getTxIdList(block);
System.out.println(txIdList);
```

* 获取区块的交易列表
```java
List<TransactionOuterClass.Tx> txList = BlockUtil.getTxList(block);
```

#### 2.3 其他
* 查询当前最后一个区块的区块号
```java
SdkClient sdkClient = new SdkClient(configFilePath);
long lastBlockNumber = BlockUtil.queryLastBlockNumber(sdkClient);
System.out.println(lastBlockNumber);
```

### 3. 查询交易数据
#### 3.1 查询交易
* 根据交易ID查询交易
```java
SdkClient sdkClient = new SdkClient(configFilePath);
String txID = "xxxxxxxxxxxxxxxxxxxxxx";
TransactionOuterClass.Tx tx = TxUtil.queryTxByTxID(sdkClient, txID);
```
#### 3.2 解析交易
* 获取交易的包含的键值
```java
List<Ledger.KeyValue> txKeyValues = TxUtil.getTxKeyValues(tx);
System.out.println(txKeyValues);
```
* 获取交易的ID
```java
String txID = TxUtil.getTxID(tx);
System.out.println(txID);
```

* 获取创建交易的组织
```java
String createOrg = TxUtil.getCreateOrg(tx);
System.out.println(createOrg);
```

* 获取给交易背书的组织
```java
List<String> endorsersOrg = TxUtil.getEndorsersOrg(tx);
System.out.println(endorsersOrg);
```

* 获取交易所使用的合约
```java
String contractName = TxUtil.getContractName(tx);
System.out.println(contractName);
```

#### 3.3 其他
* 根据交易ID查询交易结果
```java
SdkClient sdkClient = new SdkClient(configFilePath);
String txID = "xxxxxxxxxxxxxxxxxxxxxx";
String txResult = TxUtil.queryTxResultByTxID(sdkClient, txID);
System.out.println(txResult);
```




## 问题诊断

###示例1：背书失败
```
[info] send transaction result is: APPROVALS_VALIDATION_FAILURE
```
遇到以上错误，可能原因是发送交易时选择的组织与界面上“合约管理”中的“背书策略”不符。

应根据背书策略选择合适的组织节点，如选择了organization和organization1背书，则endorserNodes应写为：
```java
public static final String endorserNodes = "node-0.organization,node-0.organization1";
```
