package com.example;

import com.huawei.wienerchain.SdkClient;
import com.huawei.wienerchain.exception.SdkException;

import java.awt.Container;
import java.io.IOException;
import java.security.cert.CertificateException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import com.huawei.wienerchain.message.Builder.TxRawMsg;
import com.huawei.wienerchain.message.build.ContractRawMessage;
import com.huawei.wienerchain.proto.common.Message.Status;
import com.huawei.wienerchain.proto.common.Message.RawMessage;
import com.huawei.wienerchain.proto.common.Message.Response;
import com.huawei.wienerchain.proto.common.TransactionOuterClass.BlockResult;
import com.huawei.wienerchain.proto.common.TransactionOuterClass.TxResult;

import com.google.protobuf.InvalidProtocolBufferException;

import org.bouncycastle.util.encoders.Hex;

import java.util.Iterator;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
/**
 * 功能描述: 业务端发起请求
 *
 * @author author
 * @since 2021-08-30
 */
public class App {

    // yaml格式配置文件的绝对路径，需要您手动输入
    public static final String ConfigFilePath = "D:\\bcs-test\\bcs-test-87acff6f7-organization-563jmuz62-sdk.yaml";

    // 合约名称，需要您手动输入
    public static final String ContractName = "example";

    // 共识节点，需要您手动输入
    public static final String ConsensusNode = "node-0.organization-563jmuz62";

    // 背书节点，需要您手动输入。如果是多个节点，请用“,”隔开
    public static final String EndorserNodes = "node-1.organization-563jmuz62";

    // 查询数据所使用的节点，需要您手动输入
    public static final String QueryNode = "node-2.organization-563jmuz62";

    // 链ID，区块链的链ID，sdk yaml中chain_id字段的值
    public static final String ChainID = "bcs-test-87acff6f7";

    public static void main(String[] args) {
        try {
            // 为避免过多的grpc连接，导致内存消耗过大，一个客户端应该使用一个sdkClient对象
            SdkClient sdkClient = new SdkClient(ConfigFilePath);
            Contract contract = new Contract(ContractName, EndorserNodes, ConsensusNode);

            // 发送交易前启动监听线程
            ExecutorService singleThreadExecutor = Executors.newSingleThreadExecutor();
            singleThreadExecutor.execute(() -> {
                try {
                    queryHash(sdkClient);
                } catch (IOException | SdkException | CertificateException e) {
                    e.printStackTrace();
                }
            });

            // 发送交易
            int threadNum = 4;
            ExecutorService executorService = Executors.newFixedThreadPool(threadNum);
            // 仅为测试快速发送大量交易使用，实际商用场景应根据业务需求编写发送逻辑
            for (int i = 0; i < threadNum; i++) {
                executorService.execute(() -> {
                    try {
                        contract.send(sdkClient, "insert", new String[]{"a", "huaweichain"});
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                });
            }

        } catch (Exception e) {
            // System.out.println("[error] catch an exception: " + e);
            e.printStackTrace();
        }
    }

    // 发送交易之前，在一个独立的线程中启动该方法
    private static void queryHash(SdkClient sdkClient)
        throws IOException, SdkException, CertificateException {
        Iterator<RawMessage> iterator = sdkClient.getWienerChainNode(App.QueryNode).getEventAction().listen(App.ChainID);
        while (iterator.hasNext()) {
            RawMessage rawMessage = iterator.next();
            BlockResult blockResult = BlockResult.parseFrom(rawMessage.getPayload());
            System.out.println("-------------------------------------------------------");
            for (TxResult tx : blockResult.getTxResultsList()) {
                // 交易ID，即交易哈希
                String txId = Hex.toHexString(tx.getTxHash().toByteArray());
                // 交易结果
                String statusStr = tx.getStatus().toString();  // 只有 VALID 代表交易成功
                System.out.println("交易哈希： " + txId + ", 执行结果: " + statusStr);
            }
        }
    }
}